<?php
/**
 * Sibs Payment Gateway.
 */

use Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType;
use Sibs2\sibsConstants;
use Sibs2\sibsFunctions;

defined('ABSPATH') || exit;

/**
 * Sibs payment method integration.
 *
 * @since 1.5.0
 */
final class WC_Sibs_Blocks_Support extends AbstractPaymentMethodType
{
    /**
     * Name of the payment method.
     *
     * @var string
     */
    // protected $name = 'sibs_stg_cc';
    protected $name;

    public function __construct($method_id)
    {
        $this->name = $method_id;
    }

    /**
     * Initializes the payment method type.
     */
    public function initialize()
    {
        $this->settings = get_option('woocommerce_' . $this->get_name() . '_settings', null);
    }

    /**
     * Returns if this payment method should be active. If false, the scripts will not be enqueued.
     *
     * @return bool
     */
    public function is_active()
    {
        $payment_gateways_class = WC()->payment_gateways();
        $payment_gateways       = $payment_gateways_class->payment_gateways();

        return $payment_gateways[$this->get_name()]->is_available();
    }

    /**
     * Returns an array of scripts/handles to be registered for this payment method.
     *
     * @return array
     */
    public function get_payment_method_script_handles()
    {
        $asset_path   = WC_GATEWAY_SIBS_PATH . '/build/' . $this->get_name() . '.asset.php';
        $version      = SIBS_VERSION;
        $dependencies = [];

        if (file_exists($asset_path)) {
            $asset        = require $asset_path;
            $version      = is_array($asset) && isset($asset['version'])
                ? $asset['version']
                : $version;
            $dependencies = is_array($asset) && isset($asset['dependencies'])
                ? $asset['dependencies']
                : $dependencies;
        }

        wp_register_script(
            'wc-' . $this->get_name() . '-blocks-integration',
            WC_GATEWAY_SIBS_URL . '/build/' . $this->get_name() . '.js',
            $dependencies,
            $version,
            true
        );
        wp_set_script_translations(
            'wc-' . $this->get_name() . '-blocks-integration',
            'woocommerce-gateway-sibs'
        );

        return ['wc-' . $this->get_name() . '-blocks-integration'];
    }

    /**
     * Returns an array of key=>value pairs of data made available to the payment methods script.
     *
     * @return array
     */
    public function get_payment_method_data()
    {
        $payment_gateways = WC()->payment_gateways->payment_gateways();
        $logos            = [];

        if (isset($payment_gateways[$this->get_name()]->settings['card_types'])) {
            foreach ($payment_gateways[$this->get_name()]->settings['card_types'] as $logo) {
                $logos[] = str_replace('sibs_stg_', '', strtolower($logo));
            }
        } else {
            $logos[] = str_replace('sibs_stg_', '', strtolower($this->get_name()));
        }
        $settings        = get_option('woocommerce_' . sibsConstants::SIBS_PAYMENT_METHOD_ORDER_CREDIT_CARD . '_settings');
        $oneclick        = $settings['tokenization'] ?? null;
        $checkoutLabel   = '';
        $lang            = sibsFunctions::get_language();
        $pdf             = "sibs_terms_and_conditions_{$lang}.pdf";
        $siteUrl         = SIBS_PLUGIN_DIR;
        $pdfUrl          = "{$siteUrl}assets/files/$pdf";

        if ($oneclick === sibsConstants::SIBS_TOKEN_ONECLICK && get_current_user_id()) {
            $termsAndConditionsLinkLabel  = __('SIBS terms of service', sibsConstants::SIBS_TRANSLATE);
            $oneclickExplanationLinkLabel = __('OneClick', sibsConstants::SIBS_TRANSLATE);
            $oneclickExplanationLink      = "<a href='{$pdfUrl}' target='_blank'>{$oneclickExplanationLinkLabel}</a>";
            $checkoutLabel                = sprintf(
                __('I agree to the %s and will adhere to them unconditionally for %s payment', sibsConstants::SIBS_TRANSLATE),
                $termsAndConditionsLinkLabel,
                $oneclickExplanationLink,
            );
        }

        return [
            'title'               => $payment_gateways[$this->get_name()]->title,
            'description'         => $payment_gateways[$this->get_name()]->settings['payment_desc'] ?? '',
            'supports'            => $this->get_supported_features(),
            'logos'               => $logos,
            'checkoutLabel'       => $checkoutLabel,
            'logos_url'           => WC_GATEWAY_SIBS_URL . '/assets/images/',
            'logo_url'            => WC_GATEWAY_SIBS_URL . '/assets/images/' . strtolower(str_replace('sibs_stg_', '', $this->get_name())) . '.svg',
        ];
    }

    /**
     * Returns an array of supported features.
     *
     * @return string[]
     */
    public function get_supported_features()
    {
        $payment_gateways = WC()->payment_gateways->payment_gateways();

        return $payment_gateways[$this->get_name()]->supports;
    }
}
