<?php

/**
 * Sibs Plugin Installation process.
 *
 * This file is used for creating tables while installing the plugins.
 * Copyright (c) SIBS
 *
 * @located at  /
 */
require_once __DIR__ . '/vendor/autoload.php';

use Sibs2\sibsConstants;

defined('ABSPATH') || exit;

/**
 * Activation process.
 */
function sibs_activation_process($network_wide)
{
    create_sibs_tables($network_wide);
}

/**
 * Uninstallation process.
 */
function sibs_deactivation_process()
{
    // DO NOTHING
}

/**
 * Uninstallation process.
 */
function sibs_uninstallation_process()
{
    delete_sibs_options();
}

/**
 * Creates Sibs tables while activating the plugins
 * Calls from the hook "register_activation_hook".
 */
function create_sibs_tables($network_wide)
{
    global $wpdb;
    $wpdb->hide_errors();

    if ($network_wide) {
        $site_ids = (function_exists('get_sites') && function_exists('get_current_network_id')) ?
            get_sites(['fields' => 'ids', 'network_id' => get_current_network_id()]) :
            manually_get_sites();

        // Install the plugin for all these sites.
        foreach ($site_ids as $site_id) {
            switch_to_blog($site_id);
            woocommerce_sibs_install_single_site();
            restore_current_blog();
        }
    } else {
        woocommerce_sibs_install_single_site();
    }
}

function manually_get_sites()
{
    global $wpdb;

    $query    = "SELECT `blog_id` FROM `{$wpdb->blogs}` WHERE `site_id` = %s;";
    $bindings = [$wpdb->siteid];

    return $wpdb->get_col(
        $wpdb->prepare($query, $bindings)
    );
}

function woocommerce_sibs_install_single_site()
{
    require_once ABSPATH . 'wp-admin/includes/upgrade.php';

    $sibs_db_version = get_option(sibsConstants::SIBS_PLUGIN_PREFIX . '_db_version');

    if (empty($sibs_db_version) || $sibs_db_version < SIBS_VERSION) {
        createSibsTransactionTable($prefix, $charset_collate);
        createSibsRecurringTable($prefix, $charset_collate);
        createSibsTokenizationTable($prefix, $charset_collate);
        createSibsWebhookCallTable($prefix, $charset_collate);

        updateSibsTransationTable($prefix, $charset_collate);
        updatePluginWithVersionBefore_2_4_5();

        if (empty($sibs_db_version)) {
            add_option(sibsConstants::SIBS_PLUGIN_PREFIX . '_db_version', SIBS_VERSION);
        } elseif ($sibs_db_version < SIBS_VERSION) {
            update_option(sibsConstants::SIBS_PLUGIN_PREFIX . '_db_version', SIBS_VERSION);
        }
    }
}

/**
 * Deletes Sibs settings values from wp_options tables
 * Calls from the hook "register_deactivation_hook".
 */
function delete_sibs_options()
{
    global $wpdb;

    $option_name        = sibsConstants::SIBS_PLUGIN_PREFIX;
    $delete_options_sql = "DELETE
        FROM `{$wpdb->options}`
        WHERE `option_name` LIKE '%{$option_name}%';
    ";

    $wpdb->query($delete_options_sql);
}

/**
 * Create transaction table.
 **/
function createSibsTransactionTable($prefix, $charset_collate)
{
    global $wpdb;
    $prefix          = $wpdb->prefix;
    $charset_collate = $wpdb->get_charset_collate();

    $pending_status  = sibsConstants::WOOCOMMERCE_ORDER_STATUS_PENDING;
    $tablePrefix     = $prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
    $transaction_sql = "CREATE TABLE IF NOT EXISTS `{$tablePrefix}_transaction` (
        `id` BIGINT(20) UNSIGNED NOT NULL AUTO_INCREMENT,
        `customer_id` BIGINT(20) UNSIGNED DEFAULT NULL,
        `order_no` BIGINT(20) UNSIGNED NOT NULL,
        `reference_id` VARCHAR(32) NOT NULL,
        `captured_id` VARCHAR(100) NULL,
        `refunded_id` VARCHAR(100) NULL,
        `payment_id` VARCHAR(100) NOT NULL,
        `payment_type` VARCHAR(50) NOT NULL,
        `payment_brand` VARCHAR(100) NOT NULL,
        `payment_status` VARCHAR(30) NOT NULL DEFAULT '{$pending_status}',
        `date` DATETIME NOT NULL,
        `currency` CHAR(3) NOT NULL,
        `amount` DECIMAL(17,2) NOT NULL,
        `additional_information` LONGTEXT NULL,
        `refund_status` VARCHAR(100) NULL,
        `refunded_amount` DECIMAL(17,2) NOT NULL DEFAULT '0',
        `refund_information` LONGTEXT NULL,
        `capture_status` VARCHAR(100) NULL,
        `captured_amount` DECIMAL(17,2) NOT NULL DEFAULT '0',
        `capture_information` LONGTEXT NULL,
        `active` TINYINT(1) UNSIGNED NOT NULL DEFAULT '1',
        PRIMARY KEY (`id`),
    UNIQUE KEY (`order_no`),
        UNIQUE KEY (`reference_id`),
        UNIQUE KEY (`captured_id`),
        UNIQUE KEY (`refunded_id`),
        KEY (`customer_id`)
    ) $charset_collate;";

    dbDelta($transaction_sql);
}

/**
 * Create recurring table.
 **/
function createSibsRecurringTable($prefix, $charset_collate)
{
    $tablePrefix   = $prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
    $recurring_sql = "CREATE TABLE IF NOT EXISTS `{$tablePrefix}_payment_information` (
        `id` BIGINT(20) NOT NULL AUTO_INCREMENT,
        `cust_id` BIGINT(20) NOT NULL,
        `reg_id` VARCHAR(32),
        `payment_group` VARCHAR(6),
        `brand` VARCHAR(100),
        `holder` VARCHAR(100) NULL DEFAULT NULL,
        `email` VARCHAR(100) NULL DEFAULT NULL,
        `last4digits` VARCHAR(4),
        `expiry_month` VARCHAR(2),
        `expiry_year` VARCHAR(4),
        `server_mode` VARCHAR(4) NOT NULL,
        `channel_id` VARCHAR(32) NOT NULL,
        `payment_default` BOOLEAN NOT NULL DEFAULT '0',
        PRIMARY KEY (`id`),
        UNIQUE KEY(`cust_id`, `last4digits`, `expiry_month`, `expiry_year`)
    ) $charset_collate;";

    dbDelta($recurring_sql);
}

/**
 * Create webhook call table.
 **/
function createSibsWebhookCallTable($prefix, $charset_collate)
{
    $tablePrefix      = $prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
    $webhook_call_sql = "CREATE TABLE IF NOT EXISTS `{$tablePrefix}_webhook_calls` (
        `id` BIGINT(20) NOT NULL AUTO_INCREMENT,
        `notification_id` VARCHAR(255),
        `url` VARCHAR(255) NOT NULL,
        `headers` JSON,
        `payload` TEXT,
        `decoded_payload` JSON,
        `exception` TEXT,
        `arrived` INT DEFAULT 1,
        `processed_qty` INT DEFAULT 0,
        `processed_at` DATETIME,
        `created_at` TIMESTAMP NOT NULL DEFAULT NOW(),
        PRIMARY KEY (`id`)
    ) $charset_collate;";

    dbDelta($webhook_call_sql);
}

function checkColExist($tableName, $colName)
{
    global $wpdb;

    $col_result = $wpdb->get_results(
        "SELECT `COLUMN_NAME`
        FROM `INFORMATION_SCHEMA`.`COLUMNS`
        WHERE `table_name` = '$tableName'
        AND `COLUMN_NAME` = '$colName';",
        ARRAY_A
    );

    return ! empty($col_result);
}

function addColToTable($tableName, $colName, $colType = 'LONGTEXT', $colDefault = null, $colAfter = null)
{
    global $wpdb;

    $col_query = "ALTER TABLE `{$tableName}` ADD `$colName` $colType";

    if (! empty($colDefault)) {
        $col_query .= " DEFAULT $colDefault"; // If is string it has to be like '"string"'
    }

    if (! empty($colAfter)) {
        $col_query .= " AFTER `$colAfter`";
    }

    $col_query .= ';';

    $wpdb->query($col_query);
}

function updateSibsTransationTable($prefix)
{
    $tablePrefix        = $prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
    $tableName          = "{$tablePrefix}_transaction";

    $is_formConfig_col  = checkColExist($tableName, 'formConfig');

    if (empty($is_formConfig_col)) {
        addColToTable($tableName, 'formConfig', 'LONGTEXT', 'NULL', 'payment_status');
    }

    $is_formContext_col = checkColExist($tableName, 'formContext');

    if (empty($is_formContext_col)) {
        addColToTable($tableName, 'formContext', 'LONGTEXT', 'NULL', 'formConfig');
    }

    $is_signature_col   = checkColExist($tableName, 'signature');

    if (empty($is_signature_col)) {
        addColToTable($tableName, 'signature', 'LONGTEXT', 'NULL', 'formContext');
    }

    $is_spg_style_col   = checkColExist($tableName, 'spg_style');

    if (empty($is_spg_style_col)) {
        addColToTable($tableName, 'spg_style', 'LONGTEXT', 'NULL', 'signature');
    }
}

/**
 * Create tokenization table.
 **/
function createSibsTokenizationTable($prefix, $charset_collate)
{
    $tablePrefix      = $prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
    $tokenization_sql = "CREATE TABLE IF NOT EXISTS `{$tablePrefix}_card_tokens` (
        `id` BIGINT(20) NOT NULL AUTO_INCREMENT,
        `id_customer` BIGINT(20) UNSIGNED NOT NULL,
        `token_name` VARCHAR(100) NOT NULL,
        `token_type` VARCHAR(32) NOT NULL,
        `value` VARCHAR(100) NOT NULL,
        `masked_pan` VARCHAR(32) NOT NULL,
        `expire_date` VARCHAR(32) NOT NULL,
        `expire_date_example` VARCHAR(6) NOT NULL,
        PRIMARY KEY (`id`),
        UNIQUE KEY(`value`)
    ) $charset_collate;";

    dbDelta($tokenization_sql);
}

function migrate_payment_type($name)
{
    $tmp = [
        'PA' => sibsConstants::SPG_PAYMENT_TYPE_AUTH,
        'DB' => sibsConstants::SPG_PAYMENT_TYPE_PURS,
    ];

    return $tmp[$name] ?? sibsConstants::SPG_PAYMENT_TYPE_AUTH;
}

function migrate_payment_status($status, $type)
{
    $tmp = ($type == 'PA') ? [
        'wc-sibs-processing'  => sibsConstants::SIBS_ORDER_STATUS_AUTHORIZED,
        'wc-payment-accepted' => sibsConstants::SIBS_ORDER_STATUS_AUTHORIZED,
        'wc-payment-captured' => sibsConstants::WOOCOMMERCE_ORDER_STATUS_PROCESSING,
    ] : [
        'wc-sibs-processing'  => sibsConstants::WOOCOMMERCE_ORDER_STATUS_PROCESSING,
        'wc-payment-accepted' => sibsConstants::WOOCOMMERCE_ORDER_STATUS_PROCESSING,
        'wc-payment-captured' => sibsConstants::WOOCOMMERCE_ORDER_STATUS_PROCESSING,
    ];

    return $tmp[$status] ?? $status;
}

function seedPaymentInformationTable($table_name, $order)
{
    global $wpdb;

    $cust_id       = $order['cust_id'];
    $query         = "SELECT * FROM `$table_name` WHERE `cust_id` = %s";
    $query_prepare = $wpdb->prepare($query, $cust_id);
    $result        = $wpdb->get_row($query_prepare, ARRAY_A);

    if ($result === null) {
        $data        = [
            'cust_id'         => $order['cust_id'],
            'payment_group'   => $order['payment_group'],
            'brand'           => $order['payment_brand'],
            'holder'          => $order['holder'],
            'email'           => $order['email'],
            'last4digits'     => $order['last_4_digits'],
            'expiry_month'    => $order['expiry_month'],
            'expiry_year'     => $order['expiry_year'],
            'server_mode'     => $order['server_mode'],
            'channel_id'      => $order['channel_id'],
            'reg_id'          => $order['registration_id'],
            'payment_default' => $order['payment_default'],
        ];
        $dataFormat = ['%d', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%d'];

        $wpdb->insert(
            $table_name,
            $data,
            $dataFormat
        );
    }
}

function seedTransactionTable($table_name, $order)
{
    global $wpdb;

    $order_no      = $order['order_no'];
    $query         = "SELECT * FROM `$table_name` WHERE `order_no` = %s";
    $query_prepare = $wpdb->prepare($query, $order_no);
    $result        = $wpdb->get_row($query_prepare, ARRAY_A);

    if ($result === null) {
        $data        = [
            'order_no'                => $order_no,
            'customer_id'             => $order['customer_id'],
            'reference_id'            => $order['reference_id'],
            'captured_id'             => $order['captured_id'] ?? null,
            'refunded_id'             => $order['refunded_id'] ?? null,
            'payment_id'              => str_replace('sibs_', 'sibs_stg_', $order['payment_id']),
            'payment_type'            => migrate_payment_type($order['payment_type']), //PA => AUTH   DB => PURS
            'payment_brand'           => $order['payment_brand'],
            'payment_status'          => migrate_payment_status($order['payment_status'], $order['payment_type']),
            'date'                    => $order['date'],
            'currency'                => $order['currency'],
            'amount'                  => $order['amount'],
            'additional_information'  => $order['additional_information'],
            'capture_status'          => $order['capture_status'] ?? null,
            'captured_amount'         => $order['captured_amount'] ?? 0.00,
            'capture_information'     => $order['capture_information'] ?? null,
            'refund_status'           => $order['refund_status'],
            'refunded_amount'         => $order['refunded_amount'],
            'refund_information'      => $order['refund_information'],
        ];
        $dataFormat = ['%d', '%d', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s', '%s'];

        $wpdb->insert(
            $table_name,
            $data,
            $dataFormat
        );
    }
}

function updatePluginWithVersionBefore_2_4_5()
{
    global $wpdb;
    $table_prefix = $wpdb->base_prefix . 'sibs'; // old prefix

    $table_name   = "{$table_prefix}_transaction";
    $query        = $wpdb->prepare('SHOW TABLES LIKE %s', $wpdb->esc_like($table_name));

    if ($wpdb->get_var($query) == $table_name) {
        $query          = "SELECT * FROM `$table_name`";
        $results        = $wpdb->get_results($query, ARRAY_A);
        foreach ($results as $order) {
            $tablePrefix = $wpdb->prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
            $tableName   = "{$tablePrefix}_transaction";
            seedTransactionTable($tableName, $order);
        }
    }

    $table_name   = "{$table_prefix}_payment_information"; // old prefix
    $query        = $wpdb->prepare('SHOW TABLES LIKE %s', $wpdb->esc_like($table_name));

    if ($wpdb->get_var($query) == $table_name) {
        $query          = "SELECT * FROM `$table_name`";
        $results        = $wpdb->get_results($query, ARRAY_A);
        foreach ($results as $order) {
            $tablePrefix = $wpdb->prefix . sibsConstants::SIBS_PLUGIN_PREFIX;
            $tableName   = "{$tablePrefix}_payment_information";
            seedPaymentInformationTable($tableName, $order);
        }
    }
}
